//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  main.cpp
//  This is a part of the systemevents source code.
//
//  Copyright (C) 2003 ilmcuts.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include "main.hpp"
#include "events.hpp"
#include "win32wrap.hpp"
#include <strsafe.h>


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Globals
//
lssdk::WindowPtr g_pMainWnd;

const UINT lsMessages[] =
{
    LM_GETREVID,
    LM_REFRESH,
    LM_WINDOWACTIVATED,
    NULL
};


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// Error
//
void Error(const char* pszFormat, ...)
{
    char szMessage[MAX_PATH] = { 0 };
    va_list argList;
    
    va_start(argList, pszFormat);
    StringCchVPrintf(szMessage, MAX_PATH, pszFormat, argList);
    va_end(argList);
    
    MessageBox(NULL, szMessage, "systemevents.dll Error",
        MB_TOPMOST | MB_SETFOREGROUND | MB_ICONERROR);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ReadEventSettings
//
void ReadEventSettings(std::string sConfig, EventIDs uIdentifier)
{
    sConfig.insert(0, "*On");
    
    FILE* fStep = LCOpen(NULL);

    if (!fStep)
    {
        throw std::runtime_error("Could not open step.rc");
    }

    char szBuffer[MAX_LINE_LENGTH] = { 0 };
    
    while (LCReadNextConfig(fStep, sConfig.c_str(), szBuffer, MAX_LINE_LENGTH))
    {
        // skip the first token (*OnBlah)
        LPCSTR pszNextToken = szBuffer;
        GetToken(szBuffer, NULL, &pszNextToken, FALSE);

        if (pszNextToken)
        {
            g_EventMap.insert(EventMap::value_type(uIdentifier, pszNextToken));
        }
    }

    LCClose(fStep);
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ReadConfig
//
void ReadConfig()
{
    for (size_t stCounter = 0; stCounter < g_stEventTableItems; ++stCounter)
    {
        ReadEventSettings(g_EventTable[stCounter].pszName,
            g_EventTable[stCounter].tValue);
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// ClearConfig
//
void ClearConfig()
{
    g_EventMap.clear();
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// initModuleEx
//
int initModuleEx(HWND hLiteStep, HINSTANCE hInstance, LPCSTR szLSpath)
{
    using namespace lssdk;

    try
    {
        ReadConfig();

        WindowClassPtr pWindowClass(
            new WindowClass("LSSystemEventsClass", WindowProc, hInstance));

        g_pMainWnd = CreateMessageWindow(pWindowClass);

        SendMessage(GetLitestepWnd(), LM_REGISTERMESSAGE,
            (WPARAM)g_pMainWnd->GetHandle(), (LPARAM)lsMessages);
    }
    catch (std::exception& e)
    {
        Error("Error during initialization: %s", e.what());
    }

    ExecuteEvent(EVENT_START);

    return 0;
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// quitModule
//
void quitModule(HINSTANCE hInstance)
{
    try
    {
        SendMessage(GetLitestepWnd(), LM_UNREGISTERMESSAGE,
            (WPARAM)g_pMainWnd->GetHandle(), (LPARAM)lsMessages);

        g_pMainWnd.reset();

        ExecuteEvent(EVENT_QUIT);
        ClearConfig();
    }
    catch (std::exception& e)
    {
        // this should NOT happen... it implies an incomplete shutdown.
        // of course the shared_ptrs take care of most of it anyway
        Error("Error during shutdown: %s", e.what());
    }
}


//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
// DllMain
//
// DLL entry point
//
BOOL APIENTRY DllMain(HINSTANCE hInstance, DWORD dwReason,
                      LPVOID /* pvReserved */)
{
    switch (dwReason)
    {
        case DLL_PROCESS_ATTACH:
        {
            DisableThreadLibraryCalls(hInstance);
        }
        break;
        
        default:
        break;
    }
    
    return TRUE;
}
