#ifndef LSSDK_WIN32WRAP_HPP_INCLUDED
#define LSSDK_WIN32WRAP_HPP_INCLUDED

#if _MSC_VER > 1020
#  pragma once
#endif // _MSC_VER

//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
//
//  win32wrap.hpp
//  This is a part of the LS C/C++ SDK source code.
//
//  Copyright (C) 2003 ilmcuts.
//
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

#include <windows.h>
#include <stdexcept>
#include <string>
#include <boost/shared_ptr.hpp>
#include <boost/utility.hpp>


namespace lssdk
{
    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //
    // exception class win32_error
    //
    class win32_error : public std::runtime_error
    {
        mutable std::string m_sWhat;
        DWORD m_dwCode;

    public:
        win32_error() : runtime_error(""), m_dwCode(GetLastError()) { }
        win32_error(DWORD dwCode) : runtime_error(""), m_dwCode(dwCode) { }

        virtual ~win32_error() { }

        virtual const char* what() const
        {
            if (m_sWhat.empty())
            {
                LPVOID lpBuffer = NULL;

                if (FormatMessage(
                    FORMAT_MESSAGE_FROM_SYSTEM |
                    FORMAT_MESSAGE_IGNORE_INSERTS |
                    FORMAT_MESSAGE_ALLOCATE_BUFFER,
                    NULL,
                    m_dwCode,
                    0,
                    (LPSTR)&lpBuffer,
                    0,
                    NULL
                    ))
                {
                    m_sWhat.assign((LPCSTR)lpBuffer);
                    LocalFree(lpBuffer);
                }
            }

            return m_sWhat.c_str();
        }
    };


    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //
    // class WindowClass
    //
    class WindowClass : boost::noncopyable
    {
        HINSTANCE m_hInstance;
        ATOM m_Atom;

    public:
        WindowClass(const std::string& sName, WNDPROC pWndProc,
            HINSTANCE hInstance) : m_hInstance(hInstance)
        {
            ASSERT(!sName.empty());
            ASSERT(!IsBadCodePtr(pWndProc));
            ASSERT(hInstance != NULL);

            WNDCLASSEX wc = { 0 };
            wc.cbSize = sizeof(WNDCLASSEX);
            wc.lpfnWndProc = pWndProc;
            wc.hInstance = hInstance;
            wc.lpszClassName = sName.c_str();

            m_Atom = RegisterClassEx(&wc);

            if (!m_Atom)
            {
                throw win32_error();
            }
        }
        
        ~WindowClass()
        {
            if (m_Atom)
            {
                UnregisterClass((LPCSTR)m_Atom, m_hInstance);
            }
        }

        HINSTANCE GetInstance() const { return m_hInstance; }
        ATOM GetAtom() const { return m_Atom; }
    };

    typedef boost::shared_ptr<WindowClass> WindowClassPtr;


    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //
    // class WindowImpl
    //
    class Window : boost::noncopyable
    {
        HWND m_hWnd;
        WindowClassPtr m_pWindowClass;
    
    public:
        Window(WindowClassPtr pWindowClass, HWND hWnd)
            : m_pWindowClass(pWindowClass)
        {
            ASSERT(pWindowClass);
            ASSERT(hWnd != NULL);
            
            m_hWnd = hWnd;
        }

        ~Window()
        {
            DestroyWindow(m_hWnd);
        }

        HWND GetHandle() const { return m_hWnd; }
    };


    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //
    // typedefs
    //
    typedef boost::shared_ptr<Window> WindowPtr;



    //=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
    //
    // CreateMessageWindow
    //
    inline WindowPtr CreateMessageWindow(WindowClassPtr pWindowClass)
    {
        ASSERT(pWindowClass);

        HWND hWnd = CreateWindowEx(
            WS_EX_NOPARENTNOTIFY | WS_EX_TOOLWINDOW,
            (LPCSTR)pWindowClass->GetAtom(), NULL,
            WS_POPUP,
            0, 0, 0, 0,
            NULL, NULL,
            pWindowClass->GetInstance(),
            NULL);

        if (!hWnd)
        {
            throw win32_error();
        }

        return WindowPtr(new Window(pWindowClass, hWnd));
    }


} // namespace lssdk

#endif // !defined(LSSDK_WIN32WRAP_HPP_INCLUDED)
